@@ -71,16 +71,21 @@ class HostsEntry(object):
 
     def __repr__(self):
         return "HostsEntry(entry_type=\'{0}\', address=\'{1}\', " \
-               "comment={2}, names={3})".format(self.entry_type,
-                                                self.address,
-                                                self.comment,
-                                                self.names)
+               "names={2}, comment=\'{3}\')".format(
+                   self.entry_type,
+                   self.address,
+                   self.names,
+                   self.comment
+                   )
 
     def __str__(self):
         if self.entry_type in ('ipv4', 'ipv6'):
-            return "TYPE={0}, ADDR={1}, NAMES={2}".format(self.entry_type,
-                                                          self.address,
-                                                          " ".join(self.names))
+            return "TYPE={0}, ADDR={1}, NAMES={2}, COMMENT={3}".format(
+                self.entry_type,
+                self.address,
+                " ".join(self.names),
+                self.comment
+                )
         elif self.entry_type == 'comment':
             return "TYPE = {0}, COMMENT = {1}".format(self.entry_type, self.comment)
         elif self.entry_type == 'blank':
@@ -112,15 +117,23 @@ class HostsEntry(object):
         :param entry: A line from the hosts file
         :return: An instance of HostsEntry
         """
-        line_parts = entry.strip().split()
+        split_line = entry.split('#', 1)
+        inline_comment = None
+        if len(split_line) == 2:
+            inline_comment = split_line[1].strip()
+            line_parts = split_line[0].strip().split()
+        else:
+            line_parts = entry.strip().split()
         if is_ipv4(line_parts[0]) and valid_hostnames(line_parts[1:]):
             return HostsEntry(entry_type='ipv4',
                               address=line_parts[0],
-                              names=line_parts[1:])
+                              names=line_parts[1:],
+                              comment=inline_comment)
         elif is_ipv6(line_parts[0]) and valid_hostnames(line_parts[1:]):
             return HostsEntry(entry_type='ipv6',
                               address=line_parts[0],
-                              names=line_parts[1:])
+                              names=line_parts[1:],
+                              comment=inline_comment)
         else:
             return False
 
@@ -174,7 +187,7 @@ class Hosts(object):
         else:
             return '/etc/hosts'
 
-    def write(self, path=None):
+    def write(self, path=None, mode='w'):
         """
         Write all of the HostsEntry instances back to the hosts file
         :param path: override the write path
@@ -190,7 +203,7 @@ class Hosts(object):
         else:
             output_file_path = self.hosts_path
         try:
-            with open(output_file_path, 'w') as hosts_file:
+            with open(output_file_path, mode) as hosts_file:
                 for written_count, line in enumerate(self.entries):
                     if line.entry_type == 'comment':
                         hosts_file.write(line.comment + "\n")
@@ -200,17 +213,21 @@ class Hosts(object):
                         blanks_written += 1
                     if line.entry_type == 'ipv4':
                         hosts_file.write(
-                            "{0}\t{1}\n".format(
+                            "{0}\t{1}{2}\n".format(
                                 line.address,
                                 ' '.join(line.names),
+                                " # " + line.comment if line.comment else ""
                             )
                         )
                         ipv4_entries_written += 1
                     if line.entry_type == 'ipv6':
                         hosts_file.write(
-                            "{0}\t{1}\n".format(
+                            "{0}\t{1}{2}\n".format(
                                 line.address,
-                                ' '.join(line.names), ))
+                                ' '.join(line.names),
+                                " # " + line.comment if line.comment else ""
+                            )
+                        )
                         ipv6_entries_written += 1
         except:
             raise UnableToWriteHosts()
@@ -238,57 +255,62 @@ class Hosts(object):
         :param comment: A comment to search for
         :return: True if a supplied address, name, or comment is found. Otherwise, False.
         """
-        for entry in self.entries:
-            if entry.entry_type in ('ipv4', 'ipv6'):
-                if address and address == entry.address:
-                    return True
-                if names:
-                    for name in names:
-                        if name in entry.names:
-                            return True
-            elif entry.entry_type == 'comment' and entry.comment == comment:
+        for name in (names or [None]):
+            if self.find_all_matching(address=address, name=name, comment=comment):
                 return True
+
+        for entry in self.entries:
+            if entry.entry_type == 'comment' and entry.comment == comment:
+                return True
+            # elif entry.entry_type in ('ipv4', 'ipv6'):
+            #     pass # already covered above
         return False
 
-    def remove_all_matching(self, address=None, name=None):
+    def remove_all_matching(self, address=None, name=None, comment=None):
         """
         Remove all HostsEntry instances from the Hosts object
-        where the supplied ip address or name matches
+        where the supplied ip address, name or comment matches
         :param address: An ipv4 or ipv6 address
         :param name: A host name
+        :param comment: A host inline comment
         :return: None
         """
-        if self.entries:
-            if address and name:
-                func = lambda entry: not entry.is_real_entry() or (entry.address != address and name not in entry.names)
-            elif address:
-                func = lambda entry: not entry.is_real_entry() or entry.address != address
-            elif name:
-                func = lambda entry: not entry.is_real_entry() or name not in entry.names
-            else:
-                raise ValueError('No address or name was specified for removal.')
-            self.entries = list(filter(func, self.entries))
+        if address or name or comment:
+            pass
+        else:
+            raise ValueError('No address, name or comment was specified for removal.')
 
-    def find_all_matching(self, address=None, name=None):
+        result = self.find_all_matching(
+            address=address,
+            name=name,
+            comment=comment
+            )
+        self.entries = list(filter(lambda x: x not in result, self.entries))
+
+    def find_all_matching(self, address=None, name=None, comment=None):
         """
         Return all HostsEntry instances from the Hosts object
         where the supplied ip address or name matches
         :param address: An ipv4 or ipv6 address
         :param name: A host name
+        :param comment: A host inline comment
         :return: HostEntry instances
         """
         results = []
-        if self.entries:
+        if address or name or comment:
             for entry in self.entries:
                 if not entry.is_real_entry():
                     continue
-                if address and name:
-                    if address == entry.address and name in entry.names:
-                        results.append(entry)
-                elif address and address == entry.address:
-                    results.append(entry)
-                elif name in entry.names:
-                    results.append(entry)
+                if address:
+                    if address != entry.address:
+                        continue
+                if name:
+                    if name not in entry.names:
+                        continue
+                if comment:
+                    if comment != entry.comment:
+                        continue
+                results.append(entry)
         return results
 
     def import_url(self, url=None, force=None):
@@ -463,14 +485,19 @@ class Hosts(object):
                     elif entry_type == "blank":
                         self.entries.append(HostsEntry(entry_type="blank"))
                     elif entry_type in ("ipv4", "ipv6"):
-                        chunked_entry = hosts_entry.split()
+                        split_entry = hosts_entry.split('#', 1)
+                        chunked_entry = split_entry[0].split()
+                        comment = None
+                        if len(split_entry) == 2:
+                            comment = split_entry[1].strip()
                         stripped_name_list = [name.strip() for name in chunked_entry[1:]]
 
                         self.entries.append(
                             HostsEntry(
                                 entry_type=entry_type,
                                 address=chunked_entry[0].strip(),
-                                names=stripped_name_list))
+                                names=stripped_name_list,
+                                comment=comment))
         except IOError:
             return {'result': 'failed',
                     'message': 'Cannot read: {0}.'.format(self.hosts_path)}
